@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -258,6 +258,9 @@ void yf::BackendTest::process(Package &package) const
                 else
                     break;
 
+            *resp->preferredMessageSize = *req->preferredMessageSize;
+            *resp->maximumRecordSize = *req->maximumRecordSize;
+
             Session_info info;
             m_p->m_sessions.create(info, package.session());
         }
@@ -279,13 +282,11 @@ void yf::BackendTest::process(Package &package) const
                 int next_position = 0;
                 int error_code = 0;
                 std::string addinfo;
+                const char *element_set_name = 0;
                 
-                int number = 0;
-                mp::util::piggyback(*req->smallSetUpperBound,
-                                    *req->largeSetLowerBound,
-                                    *req->mediumSetPresentNumber,
-                                    result_set_size,
-                                    number);
+                Odr_int number = 0;
+                mp::util::piggyback_sr(req, result_set_size,
+                                       number, &element_set_name);
                 
                 if (number) 
                 {   // not a large set for sure 
@@ -368,6 +369,12 @@ void yf::BackendTest::process(Package &package) const
                 *resp->nextResultSetPosition = next_position;
             }
         }
+        else if (apdu_req->which == Z_APDU_close)
+        {
+            apdu_res = odr.create_close(apdu_req,
+                                        Z_Close_finished, 0);
+            package.session().close();
+        }
         else
         {
             apdu_res = odr.create_close(apdu_req,
@@ -382,6 +389,12 @@ void yf::BackendTest::process(Package &package) const
         m_p->m_sessions.release(package.session());
 }
 
+void mp::filter::BackendTest::configure(const xmlNode * ptr, bool test_only,
+                                        const char *path)
+{
+    mp::xml::check_empty(ptr);
+}
+
 static mp::filter::Base* filter_creator()
 {
     return new mp::filter::BackendTest;
