@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -28,6 +28,8 @@ namespace metaproxy_1 {
         class Filter_dl: public mp::filter::Base {
         public:
             void process(mp::Package & package) const;
+            void configure(const xmlNode * ptr, bool test_only,
+                           const char *path);
         };
     }
 }
@@ -36,6 +38,12 @@ void mp::filter::Filter_dl::process(mp::Package & package) const
 {
 }
 
+void mp::filter::Filter_dl::configure(const xmlNode * ptr, bool test_only,
+                                      const char *path)
+{
+    mp::xml::check_empty(ptr);
+}
+
 static mp::filter::Base* filter_creator()
 {
     return new mp::filter::Filter_dl;
