@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -18,6 +18,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "config.hpp"
 #include <metaproxy/xmlutil.hpp>
+#include <string.h>
 #include "router_flexml.hpp"
 #include "factory_filter.hpp"
 #include "factory_static.hpp"
@@ -26,6 +27,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <map>
 #include <list>
 #include <yaz/log.h>
+#include <yaz/xml_include.h>
 
 #include <boost/shared_ptr.hpp>
 
@@ -47,7 +49,8 @@ namespace metaproxy_1 {
         friend class RouterFleXML::Pos;
         Rep();
 
-        void base(xmlDocPtr doc, mp::FactoryFilter &factory, bool test_only);
+        void base(xmlDocPtr doc, mp::FactoryFilter &factory, bool test_only,
+                  const char *file_include_path);
 
         typedef std::map<std::string,
                          boost::shared_ptr<const mp::filter::Base > >
@@ -61,12 +64,17 @@ namespace metaproxy_1 {
 
         std::string m_dl_path;
 
-        void parse_xml_config_dom(xmlDocPtr doc, bool test_only);
+        void parse_xml_config_dom(xmlDocPtr doc, bool test_only,
+                                  const char *file_include_path);
 
         void parse_xml_filters(xmlDocPtr doc, const xmlNode *node,
-            bool test_only);
+                               bool test_only, const char *file_include_path);
+        void parse_xml_filters1(xmlDocPtr doc, const xmlNode *node,
+                                bool test_only, const char *file_include_path,
+                                Route &route);
         void parse_xml_routes(xmlDocPtr doc, const xmlNode *node,
-            bool test_only);
+                              bool test_only, const char *file_include_path);
+        void check_routes_in_filters(const xmlNode *n);
 
         bool m_xinclude;
     private:
@@ -88,10 +96,11 @@ namespace metaproxy_1 {
 
 void mp::RouterFleXML::Rep::parse_xml_filters(xmlDocPtr doc,
                                               const xmlNode *node,
-                                              bool test_only)
+                                              bool test_only,
+                                              const char *file_include_path)
 {
     unsigned int filter_nr = 0;
-    while(node && mp::xml::check_element_mp(node, "filter"))
+    while (node && mp::xml::check_element_mp(node, "filter"))
     {
         filter_nr++;
 
@@ -123,7 +132,7 @@ void mp::RouterFleXML::Rep::parse_xml_filters(xmlDocPtr doc,
         }
         mp::filter::Base* filter_base = m_factory->create(type_value);
 
-        filter_base->configure(node, test_only);
+        filter_base->configure(node, test_only, file_include_path);
 
         if (m_id_filter_map.find(id_value) != m_id_filter_map.end())
             throw mp::XMLError("Filter " + id_value + " already defined");
@@ -135,14 +144,85 @@ void mp::RouterFleXML::Rep::parse_xml_filters(xmlDocPtr doc,
     }
 }
 
+void mp::RouterFleXML::Rep::parse_xml_filters1(xmlDocPtr doc,
+                                              const xmlNode *node,
+                                              bool test_only,
+                                              const char *file_include_path,
+                                              Route &route)
+{
+    while (node)
+    {
+        if (mp::xml::is_element_mp(node, "filters"))
+        {
+            const xmlNode* node1 =
+                mp::xml::jump_to_children(node, XML_ELEMENT_NODE);
+
+            parse_xml_filters1(doc, node1, test_only, file_include_path, route);
+        }
+        else if (mp::xml::check_element_mp(node, "filter"))
+        {
+            const struct _xmlAttr *attr;
+            std::string refid_value;
+            std::string type_value;
+            for (attr = node->properties; attr; attr = attr->next)
+            {
+                std::string name = std::string((const char *) attr->name);
+                std::string value;
+            
+                if (attr->children && attr->children->type == XML_TEXT_NODE)
+                    value = std::string((const char *)attr->children->content);
+            
+                if (name == "refid")
+                    refid_value = value;
+                else if (name == "type")
+                    type_value = value;
+                else
+                    throw mp::XMLError("Only attribute 'refid' or 'type'"
+                                       " allowed for element 'filter'."
+                                       " Got " + name);
+            }
+            if (refid_value.length())
+            {
+                std::map<std::string,
+                         boost::shared_ptr<const mp::filter::Base > >::iterator it;
+                it = m_id_filter_map.find(refid_value);
+                if (it == m_id_filter_map.end())
+                    throw mp::XMLError("Unknown filter refid "
+                                       + refid_value);
+                else
+                    route.m_list.push_back(it->second);
+            }
+            else if (type_value.length())
+            {
+                if (!m_factory->exist(type_value))
+                {
+                    yaz_log(YLOG_LOG, "Loading %s (dlpath %s)",
+                            type_value.c_str(), m_dl_path.c_str());
+                    m_factory->add_creator_dl(type_value, m_dl_path);
+                }
+                mp::filter::Base* filter_base = m_factory->create(type_value);
+            
+                filter_base->configure(node, test_only, file_include_path);
+            
+                route.m_list.push_back(
+                    boost::shared_ptr<mp::filter::Base>(filter_base));
+            }
+        
+        }
+        node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
+    }
+}
+
+
 void mp::RouterFleXML::Rep::parse_xml_routes(xmlDocPtr doc,
                                              const xmlNode *node,
-                                             bool test_only)
+                                             bool test_only,
+                                             const char *file_include_path)
 {
     mp::xml::check_element_mp(node, "route");
 
     unsigned int route_nr = 0;
-    while(mp::xml::is_element_mp(node, "route"))
+    while (mp::xml::is_element_mp(node, "route"))
     {
         route_nr++;
 
@@ -166,63 +246,10 @@ void mp::RouterFleXML::Rep::parse_xml_routes(xmlDocPtr doc,
 
         Route route;
 
-        // process <filter> nodes in third level
+        // process <filter> / <filters> nodes in third level
         const xmlNode* node3 = mp::xml::jump_to_children(node, XML_ELEMENT_NODE);
+        parse_xml_filters1(doc, node3, test_only, file_include_path, route);
 
-        unsigned int filter3_nr = 0;
-        while(node3 && mp::xml::check_element_mp(node3, "filter"))
-        {
-            filter3_nr++;
-            
-            const struct _xmlAttr *attr;
-            std::string refid_value;
-            std::string type_value;
-            for (attr = node3->properties; attr; attr = attr->next)
-            {
-                std::string name = std::string((const char *) attr->name);
-                std::string value;
-                
-                if (attr->children && attr->children->type == XML_TEXT_NODE)
-                    value = std::string((const char *)attr->children->content);
-                
-                if (name == "refid")
-                    refid_value = value;
-                else if (name == "type")
-                    type_value = value;
-                else
-                    throw mp::XMLError("Only attribute 'refid' or 'type'"
-                                        " allowed for element 'filter'."
-                                        " Got " + name);
-            }
-            if (refid_value.length())
-            {
-                std::map<std::string,
-                    boost::shared_ptr<const mp::filter::Base > >::iterator it;
-                it = m_id_filter_map.find(refid_value);
-                if (it == m_id_filter_map.end())
-                    throw mp::XMLError("Unknown filter refid "
-                                        + refid_value);
-                else
-                    route.m_list.push_back(it->second);
-            }
-            else if (type_value.length())
-            {
-                if (!m_factory->exist(type_value))
-                {
-                    yaz_log(YLOG_LOG, "Loading %s (dlpath %s)",
-                            type_value.c_str(), m_dl_path.c_str());
-                    m_factory->add_creator_dl(type_value, m_dl_path);
-                }
-                mp::filter::Base* filter_base = m_factory->create(type_value);
-
-                filter_base->configure(node3, test_only);
-                
-                route.m_list.push_back(
-                    boost::shared_ptr<mp::filter::Base>(filter_base));
-            }
-            node3 = mp::xml::jump_to_next(node3, XML_ELEMENT_NODE);
-            
-        }
         std::map<std::string,RouterFleXML::Route>::iterator it;
         it = m_routes.find(id_value);
         if (it != m_routes.end())
@@ -234,13 +261,64 @@ void mp::RouterFleXML::Rep::parse_xml_routes(xmlDocPtr doc,
     }
 }
 
+void mp::RouterFleXML::Rep::check_routes_in_filters(const xmlNode *node)
+{
+    while (node)
+    {
+        if (mp::xml::is_element_mp(node, "filters"))
+        {
+            const xmlNode *n =
+                mp::xml::jump_to_children(node, XML_ELEMENT_NODE);
+            check_routes_in_filters(n);
+        }
+        else if (mp::xml::is_element_mp(node, "filter"))
+        {
+            const xmlNode *n =
+                mp::xml::jump_to_children(node, XML_ELEMENT_NODE);
+            while (n)
+            {
+                const struct _xmlAttr *attr;
+                // we assume thar that route attribute is only at one level
+                // below filter.. At least that works for multi and virt_db.
+                for (attr = n->properties; attr; attr = attr->next)
+                {
+                    if (!strcmp((const char *) attr->name, "route"))
+                    {
+                        std::string value;
+                        
+                        if (attr->children && attr->children->type == XML_TEXT_NODE)
+                            value = std::string((const char *)attr->children->content);
+       
+                        std::map<std::string,RouterFleXML::Route>::iterator it;
+                        it = m_routes.find(value);
+                        if (it == m_routes.end())
+                        {
+                            throw mp::XMLError("Route '" + value + "' does not exist");
+                        }
+                    }
+                }
+                n = mp::xml::jump_to_next(n, XML_ELEMENT_NODE);
+            }
+        }
+        node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
+    }
+}
+
 void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
-                                                 bool test_only)
+                                                 bool test_only,
+                                                 const char *file_include_path)
 {
     if (!doc)
         throw mp::XMLError("Empty XML Document");
     
     const xmlNode* root = xmlDocGetRootElement(doc);
+
+    if (file_include_path)
+    {
+        int r = yaz_xml_include_simple((xmlNode *) root, file_include_path);
+        if (r)
+            throw mp::XMLError("YAZ XML Include failed");
+    }
     
     mp::xml::check_element_mp(root,  "metaproxy");
 
@@ -255,7 +333,6 @@ void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
     if (mp::xml::check_element_mp(node, "start"))
     {
         const struct _xmlAttr *attr;
-        std::string id_value;
         for (attr = node->properties; attr; attr = attr->next)
         {
             std::string name = std::string((const char *) attr->name);
@@ -267,7 +344,7 @@ void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
             if (name == "route")
                 m_start_route = value;
             else
-                throw mp::XMLError("Only attribute start allowed"
+                throw mp::XMLError("Only attribute route allowed"
                                     " in element 'start'. Got " + name);
         }
         node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
@@ -277,7 +354,7 @@ void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
     {
         parse_xml_filters(doc, mp::xml::jump_to_children(node,
                                                          XML_ELEMENT_NODE),
-            test_only);
+                          test_only, file_include_path);
                       
         node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
     }
@@ -285,7 +362,7 @@ void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
     mp::xml::check_element_mp(node, "routes");
     
     parse_xml_routes(doc, mp::xml::jump_to_children(node, XML_ELEMENT_NODE),
-        test_only);
+                     test_only, file_include_path);
 
     node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
     if (node)
@@ -293,25 +370,50 @@ void mp::RouterFleXML::Rep::parse_xml_config_dom(xmlDocPtr doc,
         throw mp::XMLError("Unexpected element " 
                             + std::string((const char *)node->name));
     }
-}        
+
+    node = mp::xml::jump_to_children(root, XML_ELEMENT_NODE);
+    while (node)
+    {
+        if (mp::xml::is_element_mp(node, "filters"))
+            check_routes_in_filters(
+                mp::xml::jump_to_children(node,
+                                          XML_ELEMENT_NODE));
+        else if (mp::xml::is_element_mp(node, "routes"))
+        {
+            const xmlNode* n = mp::xml::jump_to_children(node,
+                                                         XML_ELEMENT_NODE);
+            while (n)
+            {
+                if (mp::xml::is_element_mp(n, "route"))
+                {
+                    check_routes_in_filters(
+                        mp::xml::jump_to_children(n, XML_ELEMENT_NODE));
+                    
+                }
+                n = mp::xml::jump_to_next(n, XML_ELEMENT_NODE);
+            }
+        }
+        node = mp::xml::jump_to_next(node, XML_ELEMENT_NODE);
+    }
+}
 
 mp::RouterFleXML::Rep::Rep() : m_xinclude(false)
 {
 }
 
 void mp::RouterFleXML::Rep::base(xmlDocPtr doc, mp::FactoryFilter &factory,
-    bool test_only)
+                                 bool test_only, const char *file_include_path)
 {
     m_factory = &factory;
-    parse_xml_config_dom(doc, test_only);
+    parse_xml_config_dom(doc, test_only, file_include_path);
     m_start_route = "start";
 }
 
 mp::RouterFleXML::RouterFleXML(xmlDocPtr doc, mp::FactoryFilter &factory,
-    bool test_only)
+                               bool test_only, const char *file_include_path)
     : m_p(new Rep)
 {
-    m_p->base(doc, factory, test_only);
+    m_p->base(doc, factory, test_only, file_include_path);
 }
 
 mp::RouterFleXML::RouterFleXML(std::string xmlconf, mp::FactoryFilter &factory,
@@ -326,7 +428,7 @@ mp::RouterFleXML::RouterFleXML(std::string xmlconf, mp::FactoryFilter &factory,
         throw mp::XMLError("xmlParseMemory failed");
     else
     {
-        m_p->base(doc, factory, test_only);
+        m_p->base(doc, factory, test_only, 0);
         xmlFreeDoc(doc);
     }
 }
@@ -384,6 +486,41 @@ mp::RouterFleXML::Pos::~Pos()
 }
 
 
+void mp::RouterFleXML::start()
+{
+    std::map<std::string,RouterFleXML::Route>::iterator route_it;
+
+    route_it = m_p->m_routes.begin();
+    while (route_it != m_p->m_routes.end())
+    {
+        RouterFleXML::Route route = route_it->second;
+
+        std::list<boost::shared_ptr<const mp::filter::Base> >::iterator it;
+
+        for (it = route.m_list.begin(); it != route.m_list.end(); it++)
+            (*it)->start();
+        route_it++;
+    }
+}
+
+void mp::RouterFleXML::stop()
+{
+    std::map<std::string,RouterFleXML::Route>::iterator route_it;
+
+    route_it = m_p->m_routes.begin();
+    while (route_it != m_p->m_routes.end())
+    {
+        RouterFleXML::Route route = route_it->second;
+
+        std::list<boost::shared_ptr<const mp::filter::Base> >::iterator it;
+
+        for (it = route.m_list.begin(); it != route.m_list.end(); it++)
+            (*it)->stop();
+        route_it++;
+    }
+}
+
+
 /*
  * Local variables:
  * c-basic-offset: 4
